; ##############################################################################
; ##############################################################################
; ##############################################################################
; WDTCSR  Watchdog Timer Control Register				
; WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0				
; WDP3 WDP2 WDP1 WDP0 Time								
;	0 	0 	0 	0 	2K 	(2048) 	cycles 16 ms			
;	0 	0 	0 	1 	4K 	(4096) 	cycles 32 ms			
;	0 	0 	1 	0 	8K 	(8192) 	cycles 64 ms			
;	0 	0 	1 	1 	16K (16384) cycles 0.125 s			
;	0 	1 	0 	0 	32K (32768) cycles 0.25 s			
; 	0 	1 	0 	1 	64K (65536) cycles 0.5 s			
; *	0 	1 	1 	0 	128K (131072)cycles 1.0 s			
;	0 	1 	1 	1 	256K (262144)cycles 2.0 s			
;	1 	0 	0 	0 	512K (524288)cycles 4.0 s			
;	1 	0 	0 	1 	1024K(1048576)cycles 8.0 s			
; Wenn WDIE=1 dann wird zuerst die WDR-ISR aufgerufen 	
; und dann das System neu gestartet						
WDR_INITIALISIERUNG:
	WDR
	LDS TEMP, WDTCSR
	ori TEMP, (1<<WDCE) | (1<<WDE)
	STS WDTCSR, TEMP
	ldi temp, (1<<WDE)| (1<<WDIE)|(0<<WDP3)|(1<<WDP2)|(1<<WDP1)|(0<<WDP0)
	STS WDTCSR, TEMP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SRAM_CLEAR:
	ldi ZL, LOW (SRAM_START)
	ldi ZH, HIGH(SRAM_START)
	ldi temp,0
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	breq SRAM_CLEAR_w		; gleich  => SPRUNG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STARTWERTE:
	; NULL						
	clr NULL
	; EINS						
	ldi temp,1
	mov EINS,temp
	; VERSION					
	ldi temp,(VERSION)
	STS(adr_VERSION),temp
	; SRAM CHECK				
	ldi temp,'S'
	STS(adr_SRAM_CHECK) ,temp
	; BILD						
	ldi temp,0b00000001
	STS(adr_TV_BILDBEREICH),temp
	STS(adr_TV_BILDBEREICH_ENDE-44),temp
	ldi temp,0b10000000
	STS(adr_TV_BILDBEREICH+44),temp
	STS(adr_TV_BILDBEREICH_ENDE),temp
	; TV_X_KORREKTUR			
	ldi temp,2
	STS(adr_TV_X_KORREKTUR),temp
	; TV_ZEILE					
	ldi temp1,LOW (-1)
	ldi temp2,HIGH(-1)
	mov TV_ZEILE_L,temp1
	mov TV_ZEILE_H,temp2
	; Zeit						
	ldi temp,0
	STS(adr_ss),temp
	ldi temp,0
	STS(adr_mm),temp
	ldi temp,0
	STS(adr_hh),temp
	; UTC						
	ldi temp,1
	STS(adr_UTC),temp
	; Kontrollbyte				
	ldi temp,'X'				; letztes SD-Card-Bytes
	STS(adr_GPS_BUFFER + 511),temp
	; Kontrollbyte				
	STS(adr_GPS_BUFFER + GPS_BUFFER_LEN +0),NULL	; NULL
	ldi temp,'G'
	STS(adr_GPS_BUFFER + GPS_BUFFER_LEN +1),temp
	ldi temp,'P'
	STS(adr_GPS_BUFFER + GPS_BUFFER_LEN +2),temp
	ldi temp,'S'
	STS(adr_GPS_BUFFER + GPS_BUFFER_LEN +3),temp
	; Kontrollbyte				
	STS(adr_SD_DATA + GPS_BUFFER_LEN +0),NULL	; NULL
	ldi temp,'S'
	STS(adr_SD_DATA + GPS_BUFFER_LEN +1),temp
	ldi temp,'D'
	STS(adr_SD_DATA + GPS_BUFFER_LEN +2),temp
	; VORTEILER					
	ldi temp,25
	STS(adr_ss_VT),temp
	; USART-ZEIGER				
	ldi ZL,low (adr_GPS_BUFFER)
	ldi ZH,high(adr_GPS_BUFFER)
	STS(adr_USART_ZEIGER_L),ZL
	STS(adr_USART_ZEIGER_H),ZH
	; FREIGABE					
	STS(adr_FREIGABE_SAVE),EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ERROR_SAVE_TEMP:
	LED_ROT_ON
	STS(adr_ERROR),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
WARNUNG_SAVE_TEMP:
	push ZL
	push ZH
	STS(adr_WARNUNG),temp
	LDS ZL,(adr_WARNUNG_ANZAHL_L)
	LDS ZH,(adr_WARNUNG_ANZAHL_H)
	adiw ZL,1
	STS(adr_WARNUNG_ANZAHL_L),ZL
	STS(adr_WARNUNG_ANZAHL_H),ZH
	pop ZH
	pop ZL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PORT CLEAR																	
PORT_CLEAR:
	ldi temp, 0b00000000	
	out DDRA, temp
	out DDRB, temp
	out DDRC, temp
	out DDRD, temp
	out PORTA,temp				
	out PORTB,temp				
	out PORTC,temp				
	out PORTD,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
REGISTER_CLEAR:
	clr R0
	clr R1
	clr R2
	clr R3
	clr R4
	clr R5
	clr R6
	clr R7
	clr R8
	clr R9
	clr R10
	clr R11
	clr R12
	clr R13
	clr R14
	clr R15
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_CHECK:
	; NULL		
	tst NULL
	brne PROGRAMM_CHECK_FEHLER
	; EINS		
	mov temp,EINS
	cpi temp,1
	brne PROGRAMM_CHECK_FEHLER
	; SRAM CHECK
	LDS temp,(adr_SRAM_CHECK)
	cpi temp,'S'
	brne PROGRAMM_CHECK_FEHLER
	ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_FEHLER:
	jmp PROGRAMM_CHECK_ERROR
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_INITIALISIERUNG:
	; auf AUSGANG								
	sbi (LED_GRUEN_DDR),(LED_GRUEN_PIN_NR)			; GRN		
	sbi (LED_GELB_DDR) ,(LED_GELB_PIN_NR)			; GELB		
	sbi (LED_ROT_DDR)  ,(LED_ROT_PIN_NR)			; ROT		
	sbi (LED_GRUEN_BIG_DDR),(LED_GRUEN_BIG_PIN_NR)	; GRN		
	sbi (LED_GELB_BIG_DDR) ,(LED_GELB_BIG_PIN_NR)	; GELB		
	sbi (LED_ROT_BIG_DDR)  ,(LED_ROT_BIG_PIN_NR)	; ROT		
	sbi (BEEP_DDR) 	   ,(BEEP_PIN_NR)			; BEEP		
	; LED off									
	BEEP_OFF
	rcall LED_OFF
	; Test										
	LED_GRUEN_ON
	call WAIT_100ms
	LED_GRUEN_OFF

	LED_GELB_ON
	call WAIT_100ms
	LED_GELB_OFF

	LED_ROT_ON
	call WAIT_100ms
	LED_ROT_OFF

	; Test										
	LED_GRUEN_BIG_ON
	call WAIT_100ms
	LED_GRUEN_BIG_OFF

	LED_GELB_BIG_ON
	call WAIT_100ms
	LED_GELB_BIG_OFF

	LED_ROT_BIG_ON
	call WAIT_100ms
	LED_ROT_BIG_OFF
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_OFF:
	LED_GRUEN_OFF
	LED_GELB_OFF
	LED_ROT_OFF
	LED_GRUEN_BIG_OFF
	LED_GELB_BIG_OFF
	LED_ROT_BIG_OFF
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_NEUSTART:
	CLI											; INTERRUPTS sperren	
	LED_ROT_ON
	LED_GRUEN_ON
	LED_GELB_ON
	LED_ROT_BIG_ON
	LED_GRUEN_BIG_ON
	LED_GELB_BIG_ON
	BEEP_ON
PROGRAMM_NEUSTART_s:
	jmp PROGRAMM_NEUSTART_s
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_STOP:
	WDR
	rjmp PROGRAMM_STOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
BEEP_KURZ:
	BEEP_ON
	call WAIT_20ms
	BEEP_OFF
ret
; ##############################################################################
BEEP:
	BEEP_ON
	call WAIT_100ms
	BEEP_OFF
ret
; ##############################################################################
BEEP_LANG:
	BEEP_ON
	call WAIT_500ms
	BEEP_OFF
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 14.11.2014																	
;																						
; 	INPUT:	TEMP1 / TEMP2 	1.Faktor													
;			TEMP3 / TEMP4	2.Faktor													
; 																						
; 	OUTPUT: TEMP1...4	Produkt															
; 																						
;	HILFSREGISTER: 	R0,R1,R2 (werden nicht gesichert !	)								
; 					temp5+6+7+8															
MULTIPLIKATION_16_16_32:
	push r2
	push temp5
	push temp6
	push temp7
	push temp8

	clr	r2
	mul	temp4,temp2		; ah * bh
	mov	temp7,r0
	mov	temp8,r1
	mul	temp3,temp1		; al * bl
	mov	temp5,r0
	mov	temp6,r1
	mul	temp4,temp1		; ah * bl
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2
	mul	temp2,temp3		; bh * al
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2


	mov temp1,temp5
	mov temp2,temp6
	mov temp3,temp7
	mov temp4,temp8

	pop temp8
	pop temp7
	pop temp6
	pop temp5
	pop r2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 11/2014																
;																				
;	Division 32 bit-Zahl durch 16 bit Zahl										
;																				
;	input/Output 	temp1 (LOW)	(0...4.294.967.295)								
;	input/Output 	temp2														
;	input/Output 	temp3														
;	input/Output 	temp4 (HIGH)												
;																				
;	input	temp5	Divisor	(LOW)	 											
;	input	temp6	Divisor	(HIGH)	 											
;																				
; Register:	R0+R1	Hilfsregister 												
;			temp	Hilfsregister 												
;																				
DIVISION_32_16:
	; Fehlerprfung		
	tst temp5
	brne FUNKTION_DIVISION_32_16_RUN
	tst temp6
	brne FUNKTION_DIVISION_32_16_RUN
	; Fehler NULL		
	clr temp1
	clr temp2
	clr temp3
	clr temp4
	ret	
;-------------------------------------------------------------------------------
FUNKTION_DIVISION_32_16_RUN:
;	push R0
;	push R1
;	push temp
	; RUNDEN halben Divisor dazuaddieren
	mov R0,temp5		; KOPIE			
	mov R1,temp6
	LSR R1				; durch 2		
	ROR r0
	add temp1,R0		; addieren		
	adc temp2,R1
	adc temp3,NULL
	adc temp4,NULL
;-------------------------------------------------------------------------------
DIVISION_32_16_NO_RUND:
	ldi     temp,32    ;32* schieben	
    clr     R0          ;Rest =0		
    clr     R1
FUNKTION_DIVISION_32_16_1:    
	lsl     temp1       ;Ein Bit aus dem Dividend ...   
    rol     temp2       ;ins Carry schieben
    rol     temp3
    rol     temp4
    rol     R0          ;Carry in den Rest schieben
    rol     R1
    brcs    FUNKTION_DIVISION_32_16_3       ;Bei berlauf von Rest, Sprung
    cp      R0,temp5       					;Solange Bits in temp1 (Rest) schieben bis >= divisor
    cpc     R1,temp6
    brcs    FUNKTION_DIVISION_32_16_2    	;Bei Rest<Divisor, die eingeschobene 0 in temp1 stehen lassen
FUNKTION_DIVISION_32_16_3:
    inc     temp1       ;Ansonsten Bit0 setzen und den Divisor vom Rest abziehen
    sub     R0,temp5
    sbc     R1,temp6
FUNKTION_DIVISION_32_16_2:
   	dec     temp       ;Das ganze 32 mal
    brne    FUNKTION_DIVISION_32_16_1
;	pop temp
;	pop R1
;	pop R0	
ret 
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_1024:
	subi temp1,BYTE1(-1024/2)
	sbci temp2,BYTE2(-1024/2)
	sbci temp3,BYTE3(-1024/2)
	sbci temp4,BYTE4(-1024/2)

	mov temp1,temp2
	mov temp2,temp3
	mov temp3,temp4
	clr temp4

	LSR temp3
	ROR temp2
	ROR temp1

	LSR temp3
	ROR temp2
	ROR temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
X_Y_CLEAR:
	clr XL
	clr XH
	clr YL
	clr YH
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZUFALLSZAHL_TEMP:
	clr ZL	
	clr ZH
	ldi temp9,255
ZUFALLSZAHL_s:
	LD R0,Z+
	add temp,R0
	dec temp9
	brne ZUFALLSZAHL_s
	STS(adr_ZUFALLSZAHL),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_NULL:
	STS(adr_ZAEHLER0),NULL
	STS(adr_ZAEHLER1),NULL
	STS(adr_ZAEHLER2),NULL
	STS(adr_ZAEHLER3),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_PLUS_EINS:
	LDS temp1,(adr_ZAEHLER0)
	LDS temp2,(adr_ZAEHLER1)
	LDS temp3,(adr_ZAEHLER2)
	LDS temp4,(adr_ZAEHLER3)
	add temp1,EINS
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
	STS(adr_ZAEHLER0),temp1
	STS(adr_ZAEHLER1),temp2
	STS(adr_ZAEHLER2),temp3
	STS(adr_ZAEHLER3),temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_NULL:
	clr SEKTOR0
	clr SEKTOR1
	clr SEKTOR2
	clr SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_PLUS_EINS:
	add SEKTOR0,EINS
	adc SEKTOR1,NULL
	adc SEKTOR2,NULL
	adc SEKTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_MINUS_EINS:
	tst SEKTOR0
	brne SEKTOR_MINUS_RUN
	tst SEKTOR1
	brne SEKTOR_MINUS_RUN
	tst SEKTOR2
	brne SEKTOR_MINUS_RUN
	tst SEKTOR3
	brne SEKTOR_MINUS_RUN
	ret
;-------------------------------------------------------------------------------
SEKTOR_MINUS_RUN:
	sub SEKTOR0,EINS
	sbc SEKTOR1,NULL
	sbc SEKTOR2,NULL
	sbc SEKTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_MAL_2:
	LSL SEKTOR0					; x2	
	ROL SEKTOR1
	ROL SEKTOR2
	ROL SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_DURCH_2:
	LSR SEKTOR3					; :2	
	ROR SEKTOR2
	ROR SEKTOR1
	ROR SEKTOR0
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_4_NACH_SEKTOR:
	MOV SEKTOR0,temp1
	MOV SEKTOR1,temp2
	MOV SEKTOR2,temp3
	MOV SEKTOR3,temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_NACH_TEMP1_4:
	MOV temp1,SEKTOR0
	MOV temp2,SEKTOR1
	MOV temp3,SEKTOR2
	MOV temp4,SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_PLUS_EINS:
	add temp1,EINS						; +1		
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MINUS_EINS:
	sub temp1,EINS						; -1		
	sbc temp2,NULL
	sbc temp3,NULL
	sbc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_2:
	LSR TEMP4					; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_2:
	LSL TEMP1					; x2	
	ROL TEMP2
	ROL TEMP3
	ROL TEMP4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_512:
	mov temp4,temp3			; 8xlinks		
	mov temp3,temp2
	mov temp2,temp1
	clr temp1				; LOW			

	LSL temp1				; LOW 1xlinks	
	ROL temp2
	ROL temp3
	ROL temp4				; HIGH			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_1000:
	clr R0
	clr R1
	clr R2
	clr R3
	ldi ZL,low (1000)
	ldi ZH,high(1000)
TEMP1_2_3_4_MAL_1000_s:
	add R0,temp1
	adc R1,temp2
	adc R2,temp3
	adc R3,temp4
	sbiw ZL,1		; -1
	tst ZL
	brne TEMP1_2_3_4_MAL_1000_s
	tst ZH
	brne TEMP1_2_3_4_MAL_1000_s
	mov temp1,R0
	mov temp2,R1
	mov temp3,R2
	mov temp4,R3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_10:
	ldi temp5,LOW (10)
	ldi temp6,HIGH(10)
	rjmp DIVISION_32_16_NO_RUND
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_512:
	mov temp1,temp2			; 8xlinks		
	mov temp2,temp3
	mov temp3,temp4
	clr temp4
	LSR TEMP4				; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_1000:
	ldi temp5,LOW (1000)
	ldi temp6,HIGH(1000)
	rjmp DIVISION_32_16
; ##############################################################################
; ##############################################################################
; ##############################################################################

