;																		
; TV-BAS (FBAS)															
; 																		
; 360x256 PIXEL Vollrafik												
; ASCII 45x18															
;																		
; ATmega1284P-22MHz														
;																		
; AVR-STUDIO 4															
; Bernhard.Erfurt@gmx.de												
; VERSION: 07/2019														
;																		

;																		
; Dieses Assembler-Programm generiert ein Fernsehbild.					
;																		
; Der Bildspeicher ergibt sich aus 360*256=92.160PIXEL=11.520 BYTE		
;																		

;																		
; links oben: 	(adr_TV_BILDBEREICH)		=0b00000001					
; rechts unten: (adr_TV_BILDBEREICH_ENDE)	=0b10000000					
;																		

;																		
; Das TV-Bild generiert ein 64s TIMER (TIMER-0),						
; als Kontrolltimer fr die Taktkorrektur dient der TIMER-2.			
;																		
; Jede der 625 TV-Zeilen wird separat in der "TIM0_COMPA"generiert und 	
; die TV-Norm eingehalten.												
;																		

;																		
; Um wertvolle Takte in der Interruptroutine zu sparen, wird die Zeit 	
; der hinteren Schwarzschulter genutzt, um jede							
; TV-Bild-Daten_Zeile auf Daten zu prfen, liegen keine DATEN vor,		
; dann wird diese Zeile nicht geschrieben (Zeile bleibt Dunkel)			
; das spart Takte !!!													
;																		

;																		
;Nicht alle nutzbaren TV-Zeilen werden abgebildet, ca. 60 TV-ZEILEN		
;bleiben ungenutzt.														
;																		

    
;																		
; Timer0 64s CTC Bildaufbau											
; Timer1 Taktkorrektur													
;																		

;																		
; GRN leuchtet: flackert, alles ok (Lebenslicht)						
; GELB leuchtet: ADC, SPI, TWI											
; ROT  leuchtet: schwerer Fehler, aber Programm luft weiter			
;																		

;																		
; Blinken die LEDs, das ist nicht gut !									
; ROT blinkt: 				Fehler im Programmcheck						
; ROT+GELB blinken: 		WDR-Zeitberschreitung						
; ROT+GELB+GRN blinken: 	unkorrekter Interruptaufruf					
;																		

; ATmega1284P 									
.include "m1284pdef.inc"						; ATmega1284p	
.include "MACROS.asm"							; Macros		
; VERSION										
.equ VERSION								= 1
; KONSTANTEN									
.equ TAKT									= 22000000	; 22 MHz	
; TV_BILD										
.equ XSITZE									= 8*45 ;(8*45*256=92.160=11.520 Bytes)	
.equ YSITZE									= 256
; GPS-BUFFER									
.equ GPS_BUFFER_LEN							=800
; USART_TIMER									
.equ USART_TIMER							=3
;===============================================================================
; ERROR-CODES																	
.equ ERROR_ZAHL								=0xAA 
.equ ERROR_CRC								=0xCC 
.equ ERROR_512								=0xEE 
;-PORT-A =======================================================================
; TV-DATA										
.equ TV_DATA_PIN_NR							= 0
.equ TV_DATA_DDR							= DDRA
.equ TV_DATA_PORT							= PORTA
;-PORT-B =======================================================================
; TV-SYNCHRON									
.equ TV_SYNC_PIN_NR							= 0
.equ TV_SYNC_DDR							= DDRB
.equ TV_SYNC_PORT							= PORTB
; BEEP											
.equ BEEP_PIN_NR							= 2
.equ BEEP_DDR								= DDRB
.equ BEEP_PORT								= PORTB
; SPI-CS										
.equ SPI_CS_PIN_NR							= 4
.equ SPI_CS_DDR								= DDRB
.equ SPI_CS_PORT							= PORTB
; SPI-MOSI										
.equ SPI_MOSI_PIN_NR						= 5
.equ SPI_MOSI_DDR							= DDRB
.equ SPI_MOSI_PORT							= PORTB
; SPI-MISO										
.equ SPI_MISO_PIN_NR						= 6
.equ SPI_MISO_DDR							= DDRB
.equ SPI_MISO_PORT							= PORTB
; SPI-SCK										
.equ SPI_SCK_PIN_NR							= 7
.equ SPI_SCK_DDR							= DDRB
.equ SPI_SCK_PORT							= PORTB
;-PORT-C =======================================================================
; LED-GRUEN-BIG									
.equ LED_GRUEN_BIG_PIN_NR					= 2
.equ LED_GRUEN_BIG_DDR						= DDRC
.equ LED_GRUEN_BIG_PORT						= PORTC
; LED-GELB-BIG									
.equ LED_GELB_BIG_PIN_NR					= 3
.equ LED_GELB_BIG_DDR						= DDRC
.equ LED_GELB_BIG_PORT						= PORTC
; LED-ROT-BIG									
.equ LED_ROT_BIG_PIN_NR						= 4
.equ LED_ROT_BIG_DDR						= DDRC
.equ LED_ROT_BIG_PORT						= PORTC
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR						= 5
.equ LED_GRUEN_DDR							= DDRC
.equ LED_GRUEN_PORT							= PORTC
; LED-GELB										
.equ LED_GELB_PIN_NR						= 6
.equ LED_GELB_DDR							= DDRC
.equ LED_GELB_PORT							= PORTC
; LED-ROT										
.equ LED_ROT_PIN_NR							= 7
.equ LED_ROT_DDR							= DDRC
.equ LED_ROT_PORT							= PORTC
;-PORT-D =======================================================================
; USART-RX										
.equ USART_RX_PIN_NR						= 0
.equ USART_RX_DDR							= DDRD
.equ USART_RX_PORT							= PORTD
; USART-TX										
.equ USART_TX_PIN_NR						= 1
.equ USART_TX_DDR							= DDRD
.equ USART_TX_PORT							= PORTD
; DREHGEBER "A"									
.equ DREH_A_PIN_NR							= 2
.equ DREH_A_PIN								= PIND
.equ DREH_A_DDR								= DDRD
.equ DREH_A_PORT							= PORTD
; DREHGEBER "B"									
.equ DREH_B_PIN_NR							= 3
.equ DREH_B_PIN								= PIND
.equ DREH_B_DDR								= DDRD
.equ DREH_B_PORT							= PORTD
; TASTE "DG" (Drehgeber)						
.equ TASTE_DG_PIN_NR						= 4
.equ TASTE_DG_PIN							= PIND
.equ TASTE_DG_DDR							= DDRD
.equ TASTE_DG_PORT							= PORTD
.equ TASTE_DG								= 4
; TASTE "A"										
.equ TASTE_A_PIN_NR							= 5
.equ TASTE_A_PIN							= PIND
.equ TASTE_A_DDR							= DDRD
.equ TASTE_A_PORT							= PORTD
.equ TASTE_A								= 1
; TASTE "B"										
.equ TASTE_B_PIN_NR							= 6
.equ TASTE_B_PIN							= PIND
.equ TASTE_B_DDR							= DDRD
.equ TASTE_B_PORT							= PORTD
.equ TASTE_B								= 2

; REGISTER										
.def RECHENREGISTER0= R0
.def RECHENREGISTER1= R1
.def ERROR			= R2
.def SEKTOR0		= R3
.def SEKTOR1		= R4
.def SEKTOR2		= R5
.def SEKTOR3		= R6
.def SREG_KOPIE		= R7	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp_KOPIE		= R8	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp1_KOPIE	= R9	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def ZL_KOPIE		= R10	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def ZH_KOPIE		= R11	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def TV_ZEILE_L 	= R12	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def TV_ZEILE_H 	= R13	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def NULL 			= R14	; ist immer NULL 	
.def EINS 			= R15	; ist immer EINS 	
.def temp 			= R16
.def temp1 			= R17
.def temp2 			= R18
.def temp3 			= R19
.def temp4 			= R20
.def temp5 			= R21
.def temp6 			= R22
.def temp7 			= R23
.def temp8 			= R24
.def temp9 			= R25
;-------------------------------------------------------------------------------
; SRAM-BELEGUNG Adr:0x0100-0x40FF(256-16.639=16383)		
.equ adr_DATEN							=SRAM_START+0
.equ adr_VERSION						=SRAM_START+0		; VERSION
.equ adr_UTC							=SRAM_START+1

.equ adr_TV_X_KORREKTUR					=SRAM_START+3


.equ adr_SEKTOR0						=SRAM_START+20
.equ adr_SEKTOR1						=SRAM_START+21
.equ adr_SEKTOR2						=SRAM_START+22
.equ adr_SEKTOR3						=SRAM_START+23

.equ adr_ZAEHLER0						=SRAM_START+24
.equ adr_ZAEHLER1						=SRAM_START+25
.equ adr_ZAEHLER2						=SRAM_START+26
.equ adr_ZAEHLER3						=SRAM_START+27

.equ adr_ss_VT							=SRAM_START+30		; Vorteiler					
.equ adr_ss								=SRAM_START+31		; Sekunden					
.equ adr_ss_OLD							=SRAM_START+32		;
.equ adr_mm								=SRAM_START+33		; Minuten					
.equ adr_hh								=SRAM_START+34		; Stunden					

.equ adr_FREIGABE_mm					=SRAM_START+36		; 
.equ adr_FREIGABE_hh					=SRAM_START+37		; 
.equ adr_FREIGABE_TAG					=SRAM_START+38		; 
.equ adr_FREIGABE_SAVE					=SRAM_START+39		; 

.equ adr_MENUE							=SRAM_START+40
.equ adr_TASTE							=SRAM_START+41
.equ adr_TASTEN_SPERRE					=SRAM_START+42
.equ adr_UHR_SYNCHRONISIEREN			=SRAM_START+43		; 1=ON
.equ adr_ERROR							=SRAM_START+44		
.equ adr_WARNUNG						=SRAM_START+45
.equ adr_WARNUNG_ANZAHL_L				=SRAM_START+46
.equ adr_WARNUNG_ANZAHL_H				=SRAM_START+47

.equ adr_TV_ZEIGER_DATA_L				=SRAM_START+50	; ZEIGER fr BILD-DATEN		
.equ adr_TV_ZEIGER_DATA_H				=SRAM_START+51

.equ adr_USART_TIMER					=SRAM_START+60
.equ adr_USART_ZEIGER_L					=SRAM_START+61
.equ adr_USART_ZEIGER_H					=SRAM_START+62
.equ adr_USART_BYTES_L					=SRAM_START+63
.equ adr_USART_BYTES_H					=SRAM_START+64
.equ adr_USART_BYTES_MAX_L				=SRAM_START+65
.equ adr_USART_BYTES_MAX_H				=SRAM_START+66

.equ adr_DREHGEBER_AKTIV				=SRAM_START+70
.equ adr_DREHGEBER_ROHBITS				=SRAM_START+71
.equ adr_DREHGEBER_BITMUSTER			=SRAM_START+72
.equ adr_DREHGEBER_OLD					=SRAM_START+73
.equ adr_DREHGEBER_WERT					=SRAM_START+74	; (0=NIX) (1=Richtung_1) (255=Richtung_2)	

.equ adr_ANALOGUHR_ss_OLD				=SRAM_START+80		; ANALOGUHR					
.equ adr_ANALOGUHR_OLD_ss				=SRAM_START+81		; ANALOGUHR					
.equ adr_ANALOGUHR_OLD_mm				=SRAM_START+82		; ANALOGUHR					
.equ adr_ANALOGUHR_OLD_hh				=SRAM_START+83		; ANALOGUHR					

.equ adr_ZUFALLSZAHL					=SRAM_START+90
.equ adr_DUMMY							=SRAM_START+91		; DUMMY				
.equ adr_DUMMY0							=SRAM_START+92		; DUMMY				
.equ adr_DUMMY1							=SRAM_START+93
.equ adr_DUMMY2							=SRAM_START+94
.equ adr_DUMMY3							=SRAM_START+95

.equ adr_ASCII_INVERS					=SRAM_START+100	; 1=INVERS					
.equ adr_CURSOR_ZEILE					=SRAM_START+101	; CURSOR TV-ZEILE vom CURSOR
.equ adr_CURSOR_SPALTE					=SRAM_START+102	; CURSOR-SPALTE (0..19)		

.equ adr_GRAFIC_PUNKT_GROESSE			=SRAM_START+103	; PUNKT-GRSSE 0...9		
.equ adr_GRAFIC_FARBE					=SRAM_START+104 ; 0=weiss 1=schwarz
.equ adr_GRAFIC_FONT					=SRAM_START+105

.equ adr_GRAFIC_KREIS_RADIUS			=SRAM_START+106		; RADIUS					
.equ adr_GRAFIC_WINKEL_L				=SRAM_START+107		; WINKEL					
.equ adr_GRAFIC_WINKEL_H				=SRAM_START+108		; WINKEL					
.equ adr_GRAFIC_LAENGE_L				=SRAM_START+109		; STRECKEN-LNGE			
.equ adr_GRAFIC_LAENGE_H				=SRAM_START+110		; STRECKEN-LNGE

.equ adr_GRAFIC_X1_L					=SRAM_START+120		; X1-KOORDINATEN
.equ adr_GRAFIC_X1_H					=SRAM_START+121
.equ adr_GRAFIC_Y1_L					=SRAM_START+122		; Y1-KOORDINATEN
.equ adr_GRAFIC_Y1_H					=SRAM_START+123
.equ adr_GRAFIC_X2_L					=SRAM_START+124		; X2-KOORDINATEN
.equ adr_GRAFIC_X2_H					=SRAM_START+125
.equ adr_GRAFIC_Y2_L					=SRAM_START+126		; Y2-KOORDINATEN
.equ adr_GRAFIC_Y2_H					=SRAM_START+127
.equ adr_GRAFIC_X3_L					=SRAM_START+128		; X3-KOORDINATEN
.equ adr_GRAFIC_X3_H					=SRAM_START+129
.equ adr_GRAFIC_Y3_L					=SRAM_START+130		; Y3-KOORDINATEN
.equ adr_GRAFIC_Y3_H					=SRAM_START+131
.equ adr_GRAFIC_X4_L					=SRAM_START+132		; X4-KOORDINATEN
.equ adr_GRAFIC_X4_H					=SRAM_START+133
.equ adr_GRAFIC_Y4_L					=SRAM_START+134		; Y4-KOORDINATEN
.equ adr_GRAFIC_Y4_H					=SRAM_START+135

.equ adr_GRAFIC_X1_LINIE_L				=SRAM_START+140		; fr LINIEN-BERCHNUNG		
.equ adr_GRAFIC_X1_LINIE_H				=SRAM_START+141
.equ adr_GRAFIC_Y1_LINIE_L				=SRAM_START+142
.equ adr_GRAFIC_Y1_LINIE_H				=SRAM_START+143
.equ adr_GRAFIC_X2_LINIE_L				=SRAM_START+144
.equ adr_GRAFIC_X2_LINIE_H				=SRAM_START+145
.equ adr_GRAFIC_Y2_LINIE_L				=SRAM_START+146
.equ adr_GRAFIC_Y2_LINIE_H				=SRAM_START+147

.equ adr_GRAFIC_LINIE_DELTA_X_L			=SRAM_START+150		; fr LINIEN-BERCHNUNG		
.equ adr_GRAFIC_LINIE_DELTA_X_H			=SRAM_START+151
.equ adr_GRAFIC_LINIE_DELTA_Y_L			=SRAM_START+152
.equ adr_GRAFIC_LINIE_DELTA_Y_H			=SRAM_START+153

.equ adr_GRAFIC_X1_KOPIE_L				=SRAM_START+160		; X1-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X1_KOPIE_H				=SRAM_START+161
.equ adr_GRAFIC_Y1_KOPIE_L				=SRAM_START+162		; Y1-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y1_KOPIE_H				=SRAM_START+163
.equ adr_GRAFIC_X2_KOPIE_L				=SRAM_START+164		; X2-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X2_KOPIE_H				=SRAM_START+165
.equ adr_GRAFIC_Y2_KOPIE_L				=SRAM_START+166		; Y2-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y2_KOPIE_H				=SRAM_START+167
.equ adr_GRAFIC_X3_KOPIE_L				=SRAM_START+168		; X3-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X3_KOPIE_H				=SRAM_START+169
.equ adr_GRAFIC_Y3_KOPIE_L				=SRAM_START+170		; Y3-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y3_KOPIE_H				=SRAM_START+171
.equ adr_GRAFIC_X4_KOPIE_L				=SRAM_START+172		; X4-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_X4_KOPIE_H				=SRAM_START+173
.equ adr_GRAFIC_Y4_KOPIE_L				=SRAM_START+174		; Y4-KOORDINATEN (KOPIE)	
.equ adr_GRAFIC_Y4_KOPIE_H				=SRAM_START+175


.equ adr_SD_PNM1						=SRAM_START+200		; Product name PNM				
.equ adr_SD_PNM2						=SRAM_START+201
.equ adr_SD_PNM3						=SRAM_START+202
.equ adr_SD_PNM4						=SRAM_START+203
.equ adr_SD_PNM5						=SRAM_START+204
.equ adr_SD_MDT_L						=SRAM_START+205		; Manufacturing date
.equ adr_SD_MDT_H						=SRAM_START+206
.equ adr_SD_PSN0						=SRAM_START+207		; Product serial number			
.equ adr_SD_PSN1						=SRAM_START+208
.equ adr_SD_PSN2						=SRAM_START+209
.equ adr_SD_PSN3						=SRAM_START+210

.equ adr_SD_SEKTOR_MAX0					=SRAM_START+211		; Secector max		
.equ adr_SD_SEKTOR_MAX1					=SRAM_START+212		; Secector max		
.equ adr_SD_SEKTOR_MAX2					=SRAM_START+213		; Secector max		
.equ adr_SD_SEKTOR_MAX3					=SRAM_START+214		; Secector max		

.equ adr_SD_CAPACITY_MB0				=SRAM_START+215		; Capacity in MB	
.equ adr_SD_CAPACITY_MB1				=SRAM_START+216		; Capacity in MB	
.equ adr_SD_CAPACITY_MB2				=SRAM_START+217		; Capacity in MB	
.equ adr_SD_CAPACITY_MB3				=SRAM_START+218		; Capacity in MB	

.equ adr_SD_SDHC_AKTIV					=SRAM_START+220		; 0=SDSC 1=SDHC		
.equ adr_SD_CSD_READ_BL_LEN				=SRAM_START+221 	; Register CSD =>READ_BL_LEN	
.equ adr_SD_CSD_C_SIZE_0				=SRAM_START+222		; Register CSD =>C_SIZE			
.equ adr_SD_CSD_C_SIZE_1				=SRAM_START+223		; Register CSD =>C_SIZE			
.equ adr_SD_CSD_C_SIZE_MULT				=SRAM_START+224 	; Register CSD =>C_SIZE_MULT	
.equ adr_SD_CSD_MULT_L					=SRAM_START+225
.equ adr_SD_CSD_MULT_H					=SRAM_START+226

.equ adr_GPS_CRC						=SRAM_START+500
.equ adr_GPS_ss							=SRAM_START+501
.equ adr_GPS_mm							=SRAM_START+502
.equ adr_GPS_hh							=SRAM_START+503
.equ adr_GPS_TAG						=SRAM_START+504
.equ adr_GPS_MONAT						=SRAM_START+505
.equ adr_GPS_JAHR						=SRAM_START+506

.equ adr_GPS_GGA_SAT_USE				=SRAM_START+510	; satellites in use			

.equ adr_GPS_RMC_WARNUNG				=SRAM_START+520	; Navigation receiver warning A = OK, V = warning

.equ adr_GPS_GSA_FIX_MODUS				=SRAM_START+530	; 1=Fix not available 2=2D 3=3D	
.equ adr_GPS_GSA_ID0					=SRAM_START+531
.equ adr_GPS_GSA_ID1					=SRAM_START+532
.equ adr_GPS_GSA_ID2					=SRAM_START+533
.equ adr_GPS_GSA_ID3					=SRAM_START+534
.equ adr_GPS_GSA_ID4					=SRAM_START+535
.equ adr_GPS_GSA_ID5					=SRAM_START+536
.equ adr_GPS_GSA_ID6					=SRAM_START+537
.equ adr_GPS_GSA_ID7					=SRAM_START+538
.equ adr_GPS_GSA_ID8					=SRAM_START+539
.equ adr_GPS_GSA_ID9					=SRAM_START+540
.equ adr_GPS_GSA_ID10					=SRAM_START+541
.equ adr_GPS_GSA_ID11					=SRAM_START+542

.equ adr_GPS_GSV_SAT_VIEW				=SRAM_START+551	; Total number of SVs in view	
.equ adr_GPS_GSV_SAT_VIEW_MAX			=SRAM_START+552
.equ adr_GPS_GSV_DATA					=SRAM_START+553	; PRN number(00-99); Elevation(00-90);Azimuth(000-359);SNR db(00-99)	

.equ adr_GPS_STATISTIK					=SRAM_START+1000

.equ adr_SD_DATA						=SRAM_START+2000	; SD-DATA (500Bytes)	
.equ adr_GPS_BUFFER						=SRAM_START+3000	; GPS-DATA(1000Bytes)	
.equ adr_TV_BILDBEREICH					=SRAM_START+4000 	;(256*360=92.160=11.520 Bytes)		
.equ adr_TV_BILDBEREICH_ENDE			=adr_TV_BILDBEREICH+(45*256)-1

.equ adr_SRAM_CHECK						=SRAM_START+16000	; SRAM CHECK muss immer "S" sein	
;-------------------------------------------------------------------------------

.cseg						;Beginn eines Code-Segmentes
.org 0						;Startadresse=0				
.org 0x0000 rjmp RESET 			 ; Reset									
.org 0x0002 rjmp INTERRUPT_ERROR ;rjmp INT0 ; IRQ0							
.org 0x0004 rjmp INTERRUPT_ERROR ;rjmp INT1 ; IRQ1							
.org 0x0006 rjmp INTERRUPT_ERROR ;rjmp INTEX2 ; IRQ2							
.org 0x0008 rjmp INTERRUPT_ERROR ;rjmp PCINT0 ; PCINT0						
.org 0x000A rjmp INTERRUPT_ERROR ;rjmp PCINT1 ; PCINT1						
.org 0x000C rjmp INTERRUPT_ERROR ;rjmp PCINT2 ; PCINT2						
.org 0x000E rjmp INTERRUPT_ERROR ;rjmp PCINT3 ; PCINT3						
.org 0x0010 jmp  WDT 		; Watchdog Timeout								
.org 0x0012 rjmp INTERRUPT_ERROR ;rjmp TIM2_COMPA ; Timer2 CompareA			
.org 0x0014 rjmp INTERRUPT_ERROR ;rjmp TIM2_COMPB ; Timer2 CompareB			
.org 0x0016 rjmp INTERRUPT_ERROR ;rjmp TIM2_OVF 	; Timer2 Overflow		
.org 0x0018 rjmp INTERRUPT_ERROR ;rjmp TIM1_CAPT ; Timer1 Capture			
.org 0x001A rjmp INTERRUPT_ERROR ;rjmp TIM1_COMPA ; Timer1 CompareA			
.org 0x001C rjmp INTERRUPT_ERROR ;rjmp TIM1_COMPB ; Timer1 CompareB			
.org 0x001E rjmp INTERRUPT_ERROR ;rjmp TIM1_OVF 	; Timer1 Overflow		
.org 0x0020 rjmp TIM0_COMPA ; Timer0 CompareA								
.org 0x0022 rjmp INTERRUPT_ERROR ;rjmp TIM0_COMPB ; Timer0 CompareB			
.org 0x0024 rjmp INTERRUPT_ERROR ;rjmp TIM0_OVF 	; Timer0 Overflow		
.org 0x0026 rjmp INTERRUPT_ERROR ;rjmp SPI_STC ; SPI Transfer Complete		
.org 0x0028 rjmp USART0_RXC ; USART0 RX Complete		
.org 0x002A rjmp INTERRUPT_ERROR ;rjmp USART0_UDRE ; USART0,UDR Empty		
.org 0x002C rjmp INTERRUPT_ERROR ;rjmp USART0_TXC ; USART0 TX Complete		
.org 0x002E rjmp INTERRUPT_ERROR ;rjmp ANA_COMP ; Analog Comparator			
.org 0x0030 rjmp INTERRUPT_ERROR ;rjmp ADC ; ADC Conversion Complete		
.org 0x0032 rjmp INTERRUPT_ERROR ;rjmp EE_RDY ; EEPROM Ready				
.org 0x0034 rjmp INTERRUPT_ERROR ;rjmp TWI ; 2-wire Serial					
.org 0x0036 rjmp INTERRUPT_ERROR ;rjmp SPM_RDY ; SPM Ready					
.org 0x0038 rjmp INTERRUPT_ERROR ;rjmp USART1_RXC ; USART1 RX Complete		
.org 0x003A rjmp INTERRUPT_ERROR ;rjmp USART1_UDRE ; USART1,UDR Empty		
.org 0x003C rjmp INTERRUPT_ERROR ;rjmp USART1_TXC ; USART1 TX Complete		
.org 0x003E rjmp INTERRUPT_ERROR ;rjmp TIM3_CAPT ; Timer3 Capture			
.org 0x0040 rjmp INTERRUPT_ERROR ;rjmp TIM3_COMPA  ; Timer3 CompareA		
.org 0x0042 rjmp INTERRUPT_ERROR ;rjmp TIM3_COMPB ; Timer3 CompareB			
.org 0x0044 rjmp INTERRUPT_ERROR ;rjmp TIM3_OVF 	; Timer3 Overflow		
;-------------------------------------------------------------------------------
; Unkorrekten Interrupt Aufruf 												
; ROT+GELB+GRN blint														
INTERRUPT_ERROR:
	CLI											; INTERRUPTS sperren	
	LED_GRUEN_OFF
	LED_GELB_OFF
	LED_ROT_OFF
	call wait_100ms
	LED_GRUEN_ON
	LED_GELB_ON
	LED_ROT_ON
	call wait_100ms
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
; WDR-Zeitberschreitung														
; ROT+GELB blint																
WDT:
	CLI											; INTERRUPTS sperren	
	ldi temp9,10
WDT_s:
	LED_GRUEN_OFF
	LED_GELB_OFF
	LED_ROT_OFF
	call wait_100ms
	LED_GELB_ON
	LED_ROT_ON
	call wait_100ms
	dec temp9
	brne WDT_s
WDT1:
	rjmp WDT1
;-------------------------------------------------------------------------------
; ROT blint																		
PROGRAMM_CHECK_ERROR:
	CLI											; INTERRUPTS sperren	
	ldi temp9,10
PROGRAMM_CHECK_ERROR_s:
	LED_GRUEN_OFF
	LED_GELB_OFF
	LED_ROT_OFF
	call wait_100ms
	LED_ROT_ON
	call wait_100ms
	dec temp9
	brne PROGRAMM_CHECK_ERROR_s
PROGRAMM_CHECK_ERROR_R:
	rjmp PROGRAMM_CHECK_ERROR_R
;-------------------------------------------------------------------------------
RESET:                        
	ldi temp, LOW(RAMEND)						; STACK initialisieren			
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
	call SRAM_CLEAR								; SRAM_CLEAR					
	call PORT_CLEAR								; PORT_CLEAR					
	call STARTWERTE								; STARTWERTE					
	call LED_INITIALISIERUNG					; LED							
	call TASTEN_INITIALISIERUNG					; Tasten						
	call DREHGEBER_INITIALISIEREN				; Drehgeber						
	call BILD_INITIALISIERUNG					; BILD-PIN						
	call WDR_INITIALISIERUNG					; WDR							
	call TIMER0_INITIALISIERUNG_CTC_64us		; TIMER							
	call TIMER2_INITIALISIERUNG_TAKTKORREKTUR	; TIMER							
	sei											; Interrupts freigeben			
; ##############################################################################
; ##############################################################################
; ##############################################################################
	LED_GRUEN_BIG_ON
	LED_GELB_BIG_ON
	LED_ROT_BIG_ON
	call WAIT_100ms								; einschwingen					
	call EEPROM_LOAD							; EEPROM						
	call EEPROM_SAVE							; EEPROM						
	call SPI_INITIALISIERUNG
	call SD_INITIALISIERUNG
	call PROGRAMM_CHECK
	call GPS_TEST
	call GRAFIC_CLEAR
	call USART_INITIALISIERUNG
	call BEEP_KURZ
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR										; Watch-Dog-RESET	
	LDS temp,(adr_USART_TIMER)
	cpi temp,1
	breq LOOP_USART
	LDS temp,(adr_FREIGABE_mm)
	tst temp
	brne LOOP_mm
	LDS temp,(adr_FREIGABE_hh)
	tst temp
	brne LOOP_hh
	LDS temp,(adr_FREIGABE_TAG)
	tst temp
	brne LOOP_TAG
;-------------------------------------------------------------------------------
	call MENUE
	call PROGRAMM_CHECK
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_mm:
	STS(adr_UHR_SYNCHRONISIEREN),EINS
	STS(adr_FREIGABE_mm),NULL
	call SD_CHECK
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_hh:
	STS(adr_FREIGABE_hh),NULL
	LED_GRUEN_BIG_OFF
	LED_GELB_BIG_OFF
	LED_ROT_BIG_OFF
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_TAG:
	STS(adr_FREIGABE_TAG),NULL
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_USART:
	call GPS_BEREINIGEN
	call GPS_COPY
;-------------------------------------------------------------------------------
	STS(adr_USART_TIMER),NULL
	ldi ZL,low (adr_GPS_BUFFER)
	ldi ZH,high(adr_GPS_BUFFER)
	STS(adr_USART_ZEIGER_L),ZL
	STS(adr_USART_ZEIGER_H),ZH
;-------------------------------------------------------------------------------
	call GPS_GGA	; GGA_SAT_USE				
	call GPS_RMC	; GPS_RMC_WARNUNG,Time,Date	
	call GPS_GSA	; GPS_GSA					
	call GPS_GSV_C	; GPS_GSV					
	call GPS_LED
	LED_GRUEN_OFF				; LED	
	LDS temp,(adr_FREIGABE_SAVE)
	tst temp
	brne LOOP_SAVE
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_SAVE:
	LDS temp,(adr_GPS_JAHR)
	cpi temp,20				; >=Jahr 2020		
	brsh LOOP_SAVE_w
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_SAVE_w:
	call GPS_GSV_C	; GPS_GSV					
	brcs LOOP_SAVE_w2
	rjmp LOOP
;-------------------------------------------------------------------------------
LOOP_SAVE_w2:
	call GPS_CALC_SD_SEKTOR_C
	call SD_SEKTOR_WRITE
	STS(adr_FREIGABE_SAVE),NULL
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "USART.asm"
.include "BILD.asm"				; BILDAUFBAU	
.include "TIMER.asm"			; TIMER			
.include "GPS.asm"				; GPS			
.include "WAIT.asm"				; WAIT			
.include "UHR.asm"				; Uhr			
.include "ASCII.asm"
.include "ANZEIGE.asm"
.include "ANZEIGE_KURZ.asm"
.include "GRAFIC.asm"
.include "GRAFIC_PUNKT.asm"
.include "GRAFIC_LINIE.asm"
.include "GRAFIC_RECHTECK.asm"
.include "GRAFIC_ZEIGER.asm"
.include "GRAFIC_UHR.asm"
.include "GRAFIC_SATELLITEN.asm"
.include "ZAHLEN.asm"
.include "DREHGEBER.asm"
.include "TASTEN.asm"
.include "TEXTE.asm"
.include "MENUE.asm"

.include "SPI.asm"
.include "SD.asm"
.include "SD_SONSTIGES.asm"
.include "SONSTIGES.asm"
.include "EEPROM.asm"
.include "TABELLE_SINUS_COSINUS.asm"
.include "TABELLE_FONT_08x12.asm"
.include "TABELLE_FONT_16x24.asm"
;-------------------------------------------------------------------------------
TEXT_BEGRUESSUNG_1:
.db " ",0
TEXT_BEGRUESSUNG_2:
.db "GPS 11.01.2020 ",0
TEXT_BEGRUESSUNG_3:
.db "SD-Card",0
TEXT_BEGRUESSUNG_4:
.db " ",0
TEXT_BEGRUESSUNG_5:
.db "Pixel",0
TEXT_BEGRUESSUNG_6:
.db " ",0
TEXT_BEGRUESSUNG_7:
.db "ASCII 45x18 (8x12) ",0
TEXT_BEGRUESSUNG_8:
.db "ASCII 22x10 (16x24)",0
TEXT_BEGRUESSUNG_9:
.db " ",0
TEXT_BEGRUESSUNG_10:
.db "ATmega1284p 22MHz",0
;-------------------------------------------------------------------------------
GPS_TESTDATEN:
.db "$GPGGA,191105.086,,,,,0,00,,,M,,M,,*7B",0,0
